/**
 *     project     PAFEE
 *      @file:     pafee_string.h
 *      @author:   Technocrats
 *      @brief:    Library Functions
 */

#ifndef _PAFEE_STRING_H_INCLUDED_
#define _PAFEE_STRING_H_INCLUDED_

#ifndef PAFEE_LIB_STRING_ENABLE

#include <string.h>

#define pafee_memcmp   memcmp
#define pafee_memcpy   memcpy
#define pafee_memmove  memmove
#define pafee_memset   memset
#define pafee_strcat   strcat
#define pafee_strncat  strncat
#define pafee_strcpy   strcpy
#define pafee_strncpy  strncpy
#define pafee_strlen   strlen
#define pafee_strcmp   strcmp
#define pafee_strstr   strstr
#define pafee_strchr   strchr
#define pafee_strrchr  strrchr

#else /* PAFEE_LIB_STRING_ENABLE */

#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief  pafee_memcmp
 * @param  pBuffer1 : pointer to buffer to compare
 * @param  pBuffer2 : pointer to another buffer to compare
 * @param  count : bytes to compare
 * @return result of comparison
 * @retval zero : pBuffer1 and pBuffer2 are same
 * @retval non zero : pBuffer1 and pBuffer2 are NOT same
 */
int pafee_memcmp(const void *, const void *, size_t);

/**
 * @brief  pafee_memcpy
 * @param  pTarget : pointer to destination buffer
 * @param  pSource : pointer to source buffer
 * @param  count : bytes to copy
 * @return (none)
 */
void *pafee_memcpy(void *, const void *, size_t);

/**
 * @brief  pafee_memmove
 * @param  pTarget : pointer to destination buffer
 * @param  pSource : pointer to source buffer
 * @param  count : bytes to move
 * @return (none)
 */
void *pafee_memmove(void *, const void *, size_t);

/**
 * @brief  pafee_memset
 * @param  pTarget : pointer to destination buffer
 * @param  value : value to set to buffer
 * @param  count : bytes to set value
 * @return (none)
 */
void *pafee_memset(void *, int, unsigned int);

/**
 * @brief  pafee_strcat
 * @param  pTarget : pointer to destination buffer
 * @return pSource : pointer to source buffer
 * @retval pointer to destination buffer
 */
char *pafee_strcat(char *, const char *);

/**
 * @brief  pafee_strncat
 * @param  pTarget : pointer to destination buffer
 * @param  pSource : pointer to source buffer
 * @param  count : bytes to concatenate
 * @return pointer to destination buffer
 */
char *pafee_strncat(char *, const char *, size_t);

/**
 * @brief  pafee_strcpy
 * @param  pTarget : pointer to destination buffer
 * @param  pSource : pointer to source buffer
 * @return pointer to destination buffer
 */
char *pafee_strcpy(char *, const char *);

/**
 * @brief  pafee_strncpy
 * @param  pTarget : pointer to destination buffer
 * @param  pSource : pointer to source buffer
 * @param  count : bytes to copy
 * @retval pointer to destination buffer
 */
char *pafee_strncpy(char *, const char *, size_t);

/**
 * @brief  pafee_strlen
 * @param  pString : buffer to string
 * @return length of the string
 */
size_t pafee_strlen(const char *);

/**
 * @brief  pafee_strcmp
 * @param  pBuffer1 : pointer to buffer to compare
 * @param  pBuffer2 : pointer to another buffer to compare
 * @return result of comparison
 * @retval zero : pBuffer1 and pBuffer2 are same
 * @retval non zero : pBuffer1 and pBuffer2 are NOT same
 */
int pafee_strcmp(const char *, const char *);

/**
 * @brief  pafee_strstr
 * @param  pString : pointer to string to be searched
 * @param  pSearch : pointer to string to search
 * @return result of search
 * @retval NULL : "search" is not found in "string"
 * @retval non NULL : pointer to head of first "search" in "string"
 */
char *pafee_strstr(const char *, const char *);

/**
 * @brief  pafee_strchr
 * @param  pString : pointer to string to be searched
 * @param  find : character to find
 * @return result of search
 * @retval NULL : "find" is not found in "string"
 * @retval non NULL : pointer to first "find" in "string"
 */
char *pafee_strchr(const char *, int);

/**
 * @brief  pafee_strrchr
 * @param  pString : pointer to string to be searched
 * @param  find : character to find
 * @return result of search
 * @retval NULL : "find" is not found in "string"
 * @retval non NULL : pointer to last "find" in "string"
 */
char *pafee_strrchr(const char *, int);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* PAFEE_LIB_STRING_ENABLE */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief  pafee_strrstr
 * @param  pString : pointer to string to be searched
 * @param  pSearch : pointer to string to search
 * @return result of search
 * @retval NULL : "search" is not found in "string"
 * @retval non NULL : pointer to head of last "search" in "string"
 */
char *pafee_strrstr(const char *, const char *);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _PAFEE_STRING_H_INCLUDED_ */
