//****************************************************
//
//    project   Pafee
//!   @file:    PafeeStack.h
//!   @author:  M.Tanaka
//!   @brief:   Stack
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _PAFEE_STACK_H_INCLUDE_
#define _PAFEE_STACK_H_INCLUDE_

#include <new>
#include <limits.h>
#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "PafeeList.h"
#include "pafee_string.h"

#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif //PAFEE_UNITTEST

namespace Pafee
{

//! Stack
class Stack : public Object
{

#ifdef PAFEE_STATISTICS_ENABLE
public:
	typedef struct
	{
		int addCount;
		int maxElementCount;
		int minElementSize;
		int maxElementSize;
		int presentElementCount;
	} STATISTICS;
	void GetStatistics(Stack::STATISTICS* pStatistics);
protected:
	STATISTICS m_Statistics;
#endif // PAFEE_STATISTICS_ENABLE

public:
	Stack(Allocator* pAllocator=NULL);
	Stack(const Stack& stkc, Allocator* pAllocator=NULL);
	virtual ~Stack();
	Stack& operator = (const Stack& stkc);
	//
	int GetCount(void)
	{
		return m_pList->GetCount();
	}
	bool IsEmpty()
	{
		return m_pList->IsEmpty();
	}
	void* GetWriteBuffer(
		int length // < [in] Length to set
	)
	{
		return m_pList->GetWriteBuffer(length);
	}
	bool Push(
		void* pData,  /// < [in] Data to set
		int length  /// < [in] Length to set
	)
	{
		return m_pList->AddFront(pData,length);
	}
	//No copy version
	bool Push(
		int length /// < [in] Length to set
	)
	{
		return m_pList->AddFront(length);
	}
	bool Pop(void* pBuffer, int* pLength, int bufSize=INT_MAX);
	bool Peek(
		void* pBuffer, ///< [out] Head Position Element 
		int* pLength, ///< [out] Head Position Element Size
		int bufSize=INT_MAX ///< [in] Prepared buffer sizes 
	)
	{
		return m_pList->GetFront(pBuffer,pLength,bufSize);
	}
	//No copy version
	void* Peek (
		int* pLength  /// < [out] The length that was gotten
	)
	{
		return m_pList->GetFront(pLength);
	}
	int PeekLength(void)
	{
		return m_pList->GetLengthAt(m_pList->GetFrontPosition());
	}
	void RemoveFront(void)
	{
		m_pList->RemoveFront();
	}
	void RemoveBack(void)
	{
		m_pList->RemoveBack();
	}
	void RemoveAll(void)
	{
		m_pList->RemoveAll();
	}

protected:
	List* m_pList;

#ifdef PAFEE_UNITTEST

public: // unit test function
	static void UnitTest(void);

#endif // PAFEE_UNITTEST
};

};
#endif
