//********************************************************************
//		project 	PAFEE
//
//! 	 @file: 	PafeeEventDistributor.cpp
//! 	 @author:	Takatsura Miyawaki
//! 	 @brief:	Event Distributor
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//


#include "PafeeEventDistributor.h"

using namespace Pafee;

//--------------------------------------------------------------
//! Constructor
//--------------------------------------------------------------
EventDistributor::EventDistributor()
{
	for(int i = 0; i < MAX_LISTENER; i++)
	{
		m_listeners[i] = NULL;
	}
	m_maxListener = 0;
}

//--------------------------------------------------------------
//! Destructor
//--------------------------------------------------------------
EventDistributor::~EventDistributor()
{
}

//--------------------------------------------------------------
//! Add Listener
//--------------------------------------------------------------
bool EventDistributor::AddListener(Listener* pListener)
{
	for(int i = 0; i < MAX_LISTENER; i++)
	{
		if(m_listeners[i] == NULL)
		{
			m_listeners[i] = pListener;
			if(i > m_maxListener)
			{
				m_maxListener = i;
			}
			return true;
		}
	}
	return false;
}

//--------------------------------------------------------------
//! Remove Listener
//--------------------------------------------------------------
bool EventDistributor::RemoveListener(Listener* pListener)
{
	int i;
	int maxListener = 0;
	bool isSuccess = false;

	for(i = 0; i < MAX_LISTENER; i++)
	{
		if(m_listeners[i] == pListener)
		{
			m_listeners[i] = NULL;
			isSuccess = true;
		}

		if(m_listeners[i])
		{
			maxListener = i;
		}
	}
	m_maxListener = maxListener;

	return isSuccess;
}

//--------------------------------------------------------------
//! Send event
//--------------------------------------------------------------
bool EventDistributor::SendEvent(
						unsigned long eventID,	///< [in] event id
						const void* pParam,		///< [in] parameter buffer
						int paramLength			///< [in] parameter length
						)
{
	for(int i = 0; i <= m_maxListener; i++)
	{
		if(m_listeners[i])
		{
			m_listeners[i]->SendEvent(eventID, pParam, paramLength);
		}
	}
	return true;
}

//--------------------------------------------------------------
//! event handler
//--------------------------------------------------------------
bool EventDistributor::OnEventReceived(
						unsigned long eventID,	///< [in] event id
						const void* pParam,		///< [in] parameter buffer
						int paramLength			///< [in] parameter length
						)
{
	// Empty
	return false;
}

