//********************************************************************
//		project 	PAFEE
//
//! 	 @file: 	PafeeListener.h
//! 	 @author:	Takatsura Miyawaki
//! 	 @brief:	Event Listener
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _LISTENER_H_INCLUDED_
#define _LISTENER_H_INCLUDED_

namespace Pafee
{


//! Event listener
class Listener
{

public:

	//! Constructor
	Listener()
	{ };

	//! Desstructor
	virtual ~Listener()
	{ };

	//! Send event
	inline virtual bool SendEvent(
							unsigned long eventID,	///< [in] event id
							const void* pParam,		///< [in] parameter buffer
							int paramLength			///< [in] parameter length
							)
	{
		return OnEventReceived(eventID, pParam, paramLength);
	};

protected:

	//! event handler
	virtual bool OnEventReceived(
							unsigned long eventID,	///< [in] event id
							const void* pParam,		///< [in] parameter buffer
							int paramLength			///< [in] parameter length
							) = 0;


};

};

#endif // _EVENT_LISTENER_H_INCLUDED_
