//********************************************************************
//
//! 	 @file: 	PafeeComboAllocator.h
//! 	 @author:	T.Miyawaki
//! 	 @brief:	Combo allocator
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _COMBO_ALLOCATOR_H_INCLUDED_
#define _COMBO_ALLOCATOR_H_INCLUDED_

#include "PafeeCommon.h"
#include "PafeeAllocator.h"
#ifdef PAFEE_STATISTICS_ENABLE
#include "PafeeStreamOutput.h"
#endif

namespace Pafee {

//! Combo allocator
class ComboAllocator : public Allocator
{

public:
	//! Constractor
	ComboAllocator();

	//! Destoractor
	virtual ~ComboAllocator();

	//! Allocate memory
	virtual void* Allocate(unsigned long size);

	//! Deallocate memory
	virtual void Deallocate(void* ptr);

	//! Check given address is in own address range
	virtual bool IsOwnAddress(void* ptr);

	//! Add allocator
	bool AddAllocator(Allocator* pAllocator, unsigned long minAllocationSize, unsigned long maxAllocationSize);

#ifdef PAFEE_STATISTICS_ENABLE
	//! Get statistics
	virtual void GetStatistics(StreamOutput& stream);
#endif

protected:
	enum{ MAX_ALLOCATOR = 10};		///< Max allocator count

	// Registerd allocators
	typedef struct _ALLOC
	{
		unsigned long minAllocationSize;	///< minimum allocation size
		unsigned long maxAllocationSize;	///< maximum allocation size
		Allocator*	pAllocator;				///< registerd allocator
	} ALLOC;
	ALLOC m_allocs[MAX_ALLOCATOR];
	ALLOC* m_pHeadAlloc;
	ALLOC* m_pTailAlloc;


private:
	//! Copy constractor
	ComboAllocator(const Allocator& src);

	//! Operator equal
	ComboAllocator& operator=(const Allocator& src);
};

};

#endif // _COMBO_ALLOCATOR_H_INCLUDED_
