//********************************************************************
//
//! 	 @file: 	PafeeAllocator.h
//! 	 @author:	T.Miyawaki
//! 	 @brief:	Memory allocator
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _STD_ALLOCATOR_H_INCLUDED_
#define _STD_ALLOCATOR_H_INCLUDED_

#include <stdlib.h>
#include "PafeeAllocator.h"

namespace Pafee {

class StdAllocator : public Allocator
{

public:
	//! Constractor
	StdAllocator()
	{ };
	
	//! Destoractor
	virtual ~StdAllocator()
	{ };
	

	//! Allocate memory
	virtual void* Allocate(unsigned long size)
	{
		return malloc(size);
	}

	//! Deallocate memory
	virtual void Deallocate(void* ptr)
	{
		free(ptr);
	}

	//! Check given address is in own address range
	virtual bool IsOwnAddress(void* ptr)
	{
		// it always returns true, if you use this with ComboAllocator, add this allocator at the end.
		return true;
	}
	
	static void* Malloc(unsigned long size)
	{
		return malloc(size);
	}

	static void Free(void* ptr)
	{
		free(ptr);
	}
	
protected:

private:
private:
	//! Copy constractor
	StdAllocator(const Allocator& src);

	//! Operator equal
	StdAllocator& operator=(const Allocator& src);

};

};

#endif // _STD_ALLOCATOR_H_INCLUDED_
