//****************************************************
//
//  project     PAFEE
//! @file:      PafeeString.h
//! @author:    Xie Jian
//! @brief:     string
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#include <stdarg.h>
#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "pafee_stdio.h"
#include "pafee_stdlib.h"
#include "pafee_string.h"

#ifdef ENV_WIN32
#pragma warning(disable:4996)
#endif

#ifndef _STRING_H_INCLUDED_
#define _STRING_H_INCLUDED_

#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif

namespace Pafee
{
static const int LOCAL_BUFFER_SIZE = 32;
inline int AligmentSize(int size) {return ((((size)-1)|(((size)-1)^3))+1);};

//! String
class String : public Object
{
public:
	String(Allocator* pAllocator = NULL);
	String(const String& src, Allocator* pAllocator = NULL);
	String(const char* szSrc, Allocator* pAllocator = NULL);
	void Initialize(Allocator* pAllocator);
	~String();

	String& operator = (
		const String& src /// < [in] Copied string object
	)
	{
		this->operator =(src.m_pData);
		return *this;
	}

	String& operator = (const char* szSrc);
	String& operator = (char src);

	//operator + 
	friend String operator + (
		const String& src1, ///< [in] String object
		const String& src2  ///< [in] String object
	)
	{
		String rtn(src1);
		return (rtn += src2);
	}

	friend String operator + (
		const String& src1, /// < [in] String object
		const char* szSrc2  /// < [in] Const string
	)
	{
		String rtn(src1);
		return (rtn += szSrc2);
	}

	friend String operator + (
		const char* szSrc1, /// < [in] Const string
		const String& src2 /// < [in] String object
	)
	{
		String rtn(szSrc1);
		return (rtn += src2);
	}

	friend String operator + (
		const String& src1, /// < [in] String object
		char src2 /// < [in] Const character
	)
	{
		String rtn(src1);
		return (rtn += src2);
	}

	friend String operator + (
		char src1, /// < [in] Const character
		const String& src2 /// < [in] String object
	)
	{
		String rtn;
		rtn = src1;
		return (rtn += src2);
	}

	//operator +=
	String& operator += (
		const String& src /// < [in] Added String object
	)
	{
		this->operator +=(src.m_pData);
		return *this;
	}

	String& operator += (const char* szSrc);
	String& operator += (const char  src)
	{
		char temp[2];
		temp[0] = src;
		temp[1] = '\0';
		this->operator +=(temp);
		return *this;
	}

	friend bool operator == (
		const String& src1, /// < [in] Compared string object
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp(src1.m_pData, src2.m_pData) == 0);
	}

	friend bool operator == (
		const String& src1, /// < [in] Compared string object
		const char* szSrc2 /// < [in] Compared character strings
	)
	{
		return (pafee_strcmp(src1.m_pData, szSrc2) == 0);
	}

	friend bool operator == (
		const char* szSrc1, /// < [in] Compared character strings
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp(szSrc1, src2.m_pData) == 0);
	}

	friend bool operator == (
	const String& src1, /// < [in] Compared string object
	char src2 /// < [in] Compared character
	)
	{
		char cmp2[2];
		cmp2[0] = src2;
		cmp2[1] = '\0';
		return (pafee_strcmp(src1.m_pData, cmp2) == 0);
	}

	friend bool operator == (
		char src1, /// < [in] Compared character
		const String& src2 /// < [in] Compared string object
	)
	{
		char cmp1[2];
		cmp1[0] = src1;
		cmp1[1] = '\0';
		return (pafee_strcmp(cmp1, src2.m_pData) == 0);
	}

	friend bool operator != (
		const String& src1, /// < [in] Compared string object
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp(src1.m_pData, src2.m_pData) != 0);
	}

	friend bool operator != (
		const String& src1, /// < [in] Compared string object
		const char* szSrc2 /// < [in] Compared character strings
	)
	{
		return (pafee_strcmp(src1.m_pData, szSrc2) != 0);
	}

	friend bool operator != (
		const char* szSrc1, /// < [in] Compared character strings
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp(szSrc1, src2.m_pData) != 0);
	}

	friend bool operator != (
		const String& src1, /// < [in] Compared string object
		char src2 /// < [in] Compared character
	)
	{
		char cmp2[2];
		cmp2[0] = src2;
		cmp2[1] = '\0';
		return (pafee_strcmp(src1.m_pData, cmp2) != 0);
	}

	friend bool operator != (
		char src1, /// < [in] Compared character
		const String& src2 /// < [in] Compared string object
	)
	{
		char cmp1[2];
		cmp1[0] = src1;
		cmp1[1] = '\0';
		return (pafee_strcmp(cmp1, src2.m_pData) != 0);
	}

	friend int operator < (
		const String& src1, /// < [in] Compared string object
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp (src1.m_pData, src2.m_pData) < 0);
	}

	friend int operator < (
		const String& src1, /// < [in] Compared string object
		const char* szSrc2 /// < [in] Compared character strings
	)
	{
		return (pafee_strcmp(src1.m_pData, szSrc2) < 0);
	}

	friend int operator < (
		const char* szSrc1, /// < [in] Compared character strings
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp(szSrc1, src2.m_pData) < 0);
	}

	friend int operator < (
		const String& src1, /// < [in] Compared string object
		char src2 /// < [in] Compared character
	)
	{
		char cmp2[2];
		cmp2[0] = src2;
		cmp2[1] = '\0';
		return (pafee_strcmp(src1.m_pData, cmp2) < 0);
	}

	friend int operator < (
		char src1, /// < [in] Compared character
		const String& src2 /// < [in] Compared string object
	)
	{
		char cmp1[2];
		cmp1[0] = src1;
		cmp1[1] = '\0';
		return (pafee_strcmp(cmp1, src2.m_pData) < 0);
	}

	friend int operator > (
		const String& src1, /// < [in] Compared string object
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp(src1.m_pData, src2.m_pData) > 0);
	}

	friend int operator > (
		const String& src1, /// < [in] Compared string object
		const char* szSrc2 /// < [in] Compared character strings
	)
	{
		return (pafee_strcmp(src1.m_pData, szSrc2) > 0);
	}

	friend int operator > (
		const char* szSrc1, /// < [in] Compared character strings
		const String& src2 /// < [in] Compared character strings
	)
	{
		return (pafee_strcmp(szSrc1, src2.m_pData) > 0);
	}

	friend int operator > (
		const String& src1, /// < [in] Compared character strings
		char src2 /// < [in] Compared character
	)
	{
		char cmp2[2];
		cmp2[0] = src2;
		cmp2[1] = '\0';
		return (pafee_strcmp(src1.m_pData, cmp2) > 0);
	}

	friend int operator > (
		char src1, /// < [in] Compared character
		const String& src2 /// < [in] Compared string object
	)
	{
		char cmp1[2];
		cmp1[0] = src1;
		cmp1[1] = '\0';
		return (pafee_strcmp(cmp1, src2.m_pData) > 0);
	}

	friend int operator <= (
		const String& src1, /// < [in] Compared string object
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp(src1.m_pData, src2.m_pData ) <= 0);
	}

	friend int operator <= (
		const String& src1, /// < [in] Compared string object
		const char* szSrc2 /// < [in] Compared character strings
	)
	{
		return (pafee_strcmp(src1.m_pData, szSrc2 ) <= 0);
	}

	friend int operator <= (
		const char* szSrc1, /// < [in] Compared character strings
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp(szSrc1, src2.m_pData ) <= 0);
	}

	friend int operator <= (
		const String& src1, /// < [in] Compared string object
		char src2 /// < [in] Compared character
	)
	{
		char cmp2[2];
		cmp2[0] = src2;
		cmp2[1] = '\0';
		return (pafee_strcmp(src1.m_pData, cmp2) <= 0);
	}

	friend int operator <= (
		char src1, /// < [in] Compared character
		const String& src2 /// < [in] Compared string object
	)
	{
		char cmp1[2];
		cmp1[0] = src1;
		cmp1[1] = '\0';
		return (pafee_strcmp(cmp1, src2.m_pData) <= 0);
	}

	friend int operator >= (
		const String& src1, /// < [in] Compared string object
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp(src1.m_pData, src2.m_pData) >= 0);
	}

	friend int operator >= (
		const String& src1, /// < [in] Compared string object
		const char* szSrc2 /// < [in] Compared character strings
	)
	{
		return (pafee_strcmp(src1.m_pData, szSrc2) >= 0);
	}

	friend int operator >= (
		const char* szSrc1, /// < [in] Compared character strings
		const String& src2 /// < [in] Compared string object
	)
	{
		return (pafee_strcmp(szSrc1, src2.m_pData) >= 0);
	}

	friend int operator >= (
		const String& src1, /// < [in] Compared string object
		char src2 /// < [in] Compared character
	)
	{
		char cmp2[2];
		cmp2[0] = src2;
		cmp2[1] = '\0';
		return (pafee_strcmp(src1.m_pData, cmp2) >= 0);
	}

	friend int operator >= (
		char src1, /// < [in] Compared character
		const String& src2 /// < [in] Compared string object
	)
	{
		char cmp1[2];
		cmp1[0] = src1;
		cmp1[1] = '\0';
		return (pafee_strcmp(cmp1, src2.m_pData) >= 0);
	}

	char operator [] (
		int index /// < [in] index
	)
	{
		return m_pData[index];
	}

	friend int Compare(
		const String& src1, /// < [in] Compared string object
		const String& src2 /// < [in] Compared string object
	)
	{
		return pafee_strcmp(src1.m_pData, src2.m_pData);
	}

	friend int Compare(
		const String& src1, /// < [in] Compared string object
		const char* szSrc2 /// < [in] Compared character strings
	)
	{
		return pafee_strcmp(src1.m_pData, szSrc2) ;
	}

	friend int Compare(
		const char* szSrc1, /// < [in] Compared character strings
		const String& src2 /// < [in] Compared string object
	)
	{
		return pafee_strcmp(szSrc1, src2.m_pData);
	}

	friend int Compare(
		const String& src1, /// < [in] Compared string object
		char src2 /// < [in] Compared character
	)
	{
		char cmp2[2];
		cmp2[0] = src2;
		cmp2[1] = '\0';
		return pafee_strcmp( src1.m_pData, cmp2);
	}

	friend int Compare(
		char src1, /// < [in] Compared character
		const String& src2 /// < [in] Compared string object
	)
	{
		char cmp1[2];
		cmp1[0] = src1;
		cmp1[1] = '\0';
		return pafee_strcmp(cmp1, src2.m_pData);
	}

	friend int CompareNoCase(
		const String& src1, /// < [in] Compared string object
		const String& src2 /// < [in] Compared string object
	)
	{
		String c1 = src1;
		String c2 = src2;
		return pafee_strcmp(c1.ToLower().m_pData,c2.ToLower().m_pData);
	}

	friend int CompareNoCase(
		const String& src1, /// < [in] Compared string object
		const char* szSrc2 /// < [in] Compared character strings
	)
	{
		String c1 = src1;
		String c2 = szSrc2;
		return pafee_strcmp(c1.ToLower().m_pData, c2.ToLower().m_pData) ;
	}

	friend int CompareNoCase(
		const char* szSrc1, /// < [in] Compared character strings
		const String& src2 /// < [in] Compared string object
	)
	{
		String c1 = szSrc1;
		String c2 = src2;
		return pafee_strcmp(c1.ToLower().m_pData, c2.ToLower().m_pData) ;
	}

	friend int CompareNoCase(
		const String& src1, /// < [in] Compared string object
		char src2/// < [in] Compared character
	)
	{
		String c1 = src1;
		String c2;
		c2 = src2;
		return pafee_strcmp(c1.ToLower().m_pData, c2.ToLower().m_pData) ;
	}

	friend int CompareNoCase(
		char src1, /// < [in] Compared character
		const String& src2 /// < [in] Compared string object
	)
	{
		String c1;
		c1 = src1;
		String c2 = src2;
		return pafee_strcmp(c1.ToLower().m_pData, c2.ToLower().m_pData);
	}

	int Find(
		const String& target, /// < [in] Retrieved string object
		int from = 0 /// < [in] Position in which retrieval begins
	)
	{
		return this->Find(target.m_pData, from);
	}

	int Find(const char* szTarget, int from =0);
	int Find(
		char target, /// < [in] Retrieved character
		int from =0 /// < [in] Position in which retrieval begins
	)
	{
		char temp[2];
		temp[0] = target;
		temp[1] = '\0';
		return this->Find(temp,from);
	}

	int ReverseFind(
		const String& target /// < [in] Retrieved string object
	)
	{
		return this->ReverseFind(target.m_pData);
	}

	int ReverseFind(const char* szTarget);

	int ReverseFind(
		char targetChar /// < [in] Retrieved character
	)
	{
		char temp[2];
		temp[0] = targetChar;
		temp[1] = '\0';
		return this->ReverseFind(temp);
	}

	void Format(const char* format, ...);
	void Insert(
		int indexAt, /// < [in] Insert Position
		const String& src /// < [in] Insert String
	)
	{
		this->Insert(indexAt,src.m_pData);
	}

	void Insert(int indexAt, const char* szSrc);
	void Insert(
		int indexAt, /// < [in] Insert Position
		char src /// < [in] Insert character
	)
	{
		char temp[2];
		temp[0] = src;
		temp[1] = '\0';
		this->Insert(indexAt,temp);
	}

	void Delete(int index, int count=1);
	void DeleteLeft(
		int count /// < [in] Delete size
	)
	{
		this->Delete(0,count);
	}

	void DeleteRight(int count);
	String Substring(int from, int count);
	String Substring(
		int from /// <[in] Starting position to sub string
	)
	{
		return this->Substring(from,(m_length - from));
	}

	String Left(
		int count /// < [in] Sub string size
	)
	{
		return this->Substring(0,count);
	}

	String Right(int count);
	String& Trim(
		const char* szTarget = " \t" /// < [in] Removed character strings
	)
	{
		TrimLeft(szTarget);
		TrimRight(szTarget);
		return *this;
	}

	String& Trim(
		char target /// < [in] Removed character
	)
	{
		TrimLeft(target);
		TrimRight(target);
		return *this;
	}

	String& TrimLeft(const char* szTarget = " \t");
	String& TrimLeft(char target);
	String& TrimRight(const char* szTarget = " \t");
	String& TrimRight(char target);
	String& ToLower(void);
	String& ToUpper(void);
	char* GetBuffer() const {return m_pData;};
	int GetLength() const {return m_length;};
	bool IsEmpty(){return (m_length == 0);};
private: 
	void GetAllocateSize(int mLength);
	int GetNewLen(const char *format,va_list ap);
private:
	char* m_pData;
	char m_localBuffer[LOCAL_BUFFER_SIZE];
	int m_length;
	int m_allocateSize;

#ifdef PAFEE_UNITTEST
public: // unit test function
	static void UnitTest(void);
#endif // PAFEE_UNITTEST

};

}
#endif //_STRING_H_INCLUDED_

