//****************************************************
//
//      project     PAFEE
//!      @file:     FileCollector.h
//!      @author:   Technocrats
//!      @brief:    File collector class header
//
//*****************************************************

#ifndef _FILE_COLLECTOR_H_INCLUDED
#define _FILE_COLLECTOR_H_INCLUDED


#include "define.h"
#include <string>
#include <list>
#include <stack>

using namespace std;

#define	DEFAULT_OUT_PATH		"./Output"
#define	DEFAULT_SOURCE_PATH		"../Src"
namespace Pafee{
class FileCollector{
private:
	list<string> m_fileTable; /**!< file name list */
	stack<string> m_makeList;
	string m_sourcePath; /**!< PAFEE source path */
	string m_outputPath; /**!< PAFEE output path */
	BOOL m_isOverwrite; /**!< File overwrite flag */

	/**
	 * @brief copy constractor of FileCollector Class
	 */
	FileCollector(const FileCollector& org);

	/**
	 * @brief operator: equal
	 */
	FileCollector& operator=(const FileCollector& org);

	/**
	 * @brief  Create folder. If there is, not create
	 * @param  FolderPath Creating folder path
	 * @return Create success or not
	 * @retval true Create success
	 * @retval false Createt failure
	 */
	BOOL CreateFolder(
		const string FolderPath
	);

public:

	/**
	 * @brief constractor of FileCollector Class
	 */
	FileCollector();

	/**
	 * @brief destoractor of FileCollector Class
	 */
	~FileCollector(); 

	/**
	 * @brief  Select PAFEE source folder
	 * @param  pSourcePath PAFEE source path
	 * @return non
	 */
	void SetSourcePath(
		const string sourcePath=DEFAULT_SOURCE_PATH
	);

	/**
	 * @brief  Select PAFEE output folder
	 * @param  sourcePath PAFEE source path
	 * @return non
	 */
	void SetOutputPath(
		const string outputPath=DEFAULT_OUT_PATH
	);

	/**
	 * @brief  Output files in appointed folder
	 * @param  OutputPath Output folder path
	 * @return Output success or not
	 * @retval true Output success
	 * @retval false Output failure
	 */
	BOOL PutFile(
		const list<string> fileTable
	);
};
}; // namespace Pafee
#endif // _FILE_COLLECTOR_H_INCLUDED
