//****************************************************
//
//      project     PAFEE
//!      @file:     ClassDependency.c
//!      @author:   Technocrats
//!      @brief:    Dependence definition file analysis class source
//
//*****************************************************

#include "StdAfx.h"
#include "define.h"
#include "assert.h"
#include "ClassDependency.h"
#include <string>
#include <fstream>
#include <iostream>
#include <windows.h>

#define DEPEND_FILE_MAX_LINE_STRING 1024

using namespace std;

#ifdef __cplusplus
namespace Pafee{
#endif

static const char g_delimiter[] = " \r\n";
static const char g_partition[] = ":";
static const string g_comment = "#";
static const string g_freeStanding = "CLib";
static const string g_deleteString = " \t";
static const string g_pathDelimitor = "/\\";
//static const string g_dependFileName = "pafeeConfig.depend";
static const string g_dependFileName = "pafee.depend";

/**
 * @brief constractor of ClassDependency Class
 * @param filePath Depend file path
 */
ClassDependency::ClassDependency(
	const string filePath
)
{
	this->ReadFile(filePath);
}

/**
 * @brief copy constractor of ClassDependency Class
 */
ClassDependency::ClassDependency(
	const ClassDependency& org
)
{
}

/**
 * @brief destoractor of ClassDependency Class
 */
ClassDependency::~ClassDependency(
)
{
}

/**
 * @brief operator: equal
 */
ClassDependency& ClassDependency::operator=(
	const ClassDependency& org
)
{
	return *this;
}

/**
 * @brief  Depend file read method
 * @param  filePath Depend file path
 * @return Read success or not
 * @retval TRUE Read success
 * @retval FALSE Read failure
 */
BOOL ClassDependency::ReadFile(
	const string filePath
)
{
	BOOL retval=FALSE;
	SHORT length;

	length = filePath.length();

	if(filePath.empty())
	{
		this->m_filePath = g_dependFileName;
	}
	else if((filePath[length-1] == '/') || (filePath[length-1] == '\\'))
	{
		this->m_filePath = filePath + g_dependFileName;
	}
	else
	{
		this->m_filePath = filePath;
	}

	return retval;
}

/**
 * @brief  Analyze dependency
 * @param  pClassTable Using class define table
 * @return Analysis success or not
 * @retval TRUE Analysis success
 * @retval FALSE Analysis failure
 */
BOOL ClassDependency::DefineDependency(
	const list<string> classTable,
	const BOOL isFreeStanding
)
{
	list<string>::iterator classIterator;
	list<string>::iterator useClassIterator;
	string className;
	string findString;
	string::size_type first;
	BOOL retval=TRUE;

	/* clear buffer */
	m_dependencyTable.clear();
	m_useClassList.clear();
	while(!m_findClassList.empty())
	{
		m_findClassList.pop();
	}

	m_classTable = classTable;
	classIterator = m_classTable.begin();

#if 0
	if(isFreeStanding == TRUE)
	{
		retval = ClassFileChecker(g_freeStanding);
	}
#endif

	if(retval == TRUE)
	{
		while(classIterator != m_classTable.end())
		{
			/* add use class */
			m_useClassList.push_back(*classIterator);

			/* get class name */
			first = (*classIterator).find_last_of("/\\");
			className = (*classIterator).substr(first + 1);

			/* check depend file */
			retval = ClassFileChecker(className);

			if(retval == TRUE)
			{
				/* check class folder */
				retval = ClassFolderChecker(*classIterator);
			}

			classIterator++;

			if(retval == FALSE)
			{
				break;
			}
		}
	}

	if(retval == TRUE)
	{
		while(!(m_findClassList.empty()))
		{
			findString = m_findClassList.front();
			m_findClassList.pop();
			retval = ClassFileChecker(findString);

			if(retval == FALSE)
			{
				break;
			}
		}
	}

	m_dependencyTable.unique();

#if 0
	while(!(m_dependencyTable.empty()))
	{
		findString = m_dependencyTable.front();
		m_dependencyTable.pop_front();
		cout << findString << "--\n";
	}
#endif

	return retval;
}

/**
 * @brief  Get dependency
 * @param  pDependencyTable Pointer of a buffer storing away dependence
 * @return null
 */
BOOL ClassDependency::GetDependency(
	list<string> *pDependencyTable
)
{
	BOOL retval=FALSE;

	if(pDependencyTable != NULL)
	{
		*pDependencyTable = m_dependencyTable;
		retval = TRUE;
	}

	return retval;
}

/**
 * @brief  check class use file and other class
 * @param  classNum number of PAFEE_CLASS_TABLE
 * @return Read success or not
 * @retval TRUE Read success
 */
BOOL ClassDependency::ClassFileChecker(
	const string className
)
{
	queue<string> fileQueue;
	BOOL retval=FALSE;
	ifstream ifs;
	string textLine;
	string objectLine;
	string compareString;
	string::size_type first;
	string::size_type end;
	char *pToken;
	char pTextLine[DEPEND_FILE_MAX_LINE_STRING];

	// Check depend file exist
	ifs.open(m_filePath.c_str(), ios::in);

	if(ifs)
	{
		/* check depend file */
		while(!ifs.eof())
		{
			objectLine = "\\";
			textLine = "\\";

			/* Add line until end of line is '\' */
			while(textLine[textLine.size() - 1] == '\\')
			{
				if(getline(ifs, textLine))
				{
					objectLine = objectLine.substr(0, objectLine.size() - 1) + textLine;
				}
			}

			/* All line readed */
			if(objectLine[0] == '\0')
			{
				break;
			}

			/* Copy string to char buffer */
			strcpy(pTextLine, objectLine.c_str());

			/* Sepalate [:] */
			pToken = strtok(pTextLine, g_partition);
			compareString = pToken;

			/* delete space befor and after category */
			compareString = pafee_DeleteCharacter(compareString, g_delimiter);

			/* if before[:] is class name */
			if((compareString == ("Pafee" + className + ".o")) || (compareString == (className + ".o")))
			{
				/* Token exist */
				while(pToken != NULL)
				{
					/* Sepalate .../..* */
					pToken = strtok(NULL, g_delimiter);

					if(pToken != NULL)
					{
						compareString = pToken;

						/* find last [/] or [\] */
						first = compareString.find_last_of("/\\");
						if(first == string::npos)
						{
							first = 0;
						}

						/* find last [.] */
						end = compareString.find_last_of(".");
						if(end == string::npos)
						{
							end = compareString.size();
						}

						/* insert file pass */
						if(m_dependencyTable.end() == find(m_dependencyTable.begin(), m_dependencyTable.end(), compareString))
						{
							m_dependencyTable.push_back(compareString);

#if 0
							/* if file name is ~.h */
							if(compareString.substr(end, 2) == ".h")
							{
								HeaderFileChecker(compareString);
							}
#endif
						}

						/* if file pass's class name is not exit yet */
						compareString = compareString.substr(first + 1, end - first - 1);

						/* if file path's Pafee* */
						if(compareString.substr(0, 5) == "Pafee")
						{
							compareString = compareString.substr(5, compareString.size());
						}

						if(m_useClassList.end() == find(m_useClassList.begin(), m_useClassList.end(), compareString))
						{
							m_useClassList.push_back(compareString);
							m_findClassList.push(compareString);
						}
					}
				}

				retval = TRUE;
				break;
			}
			else
			{
				/* next line */
			}
		}
	}
	else
	{
		cout << "Can not find dependency definition file\n";
	}

	if(retval == FALSE)
	{
		cout << "Can not find dependency. class: " << className << endl;
		retval = TRUE;
	}

	return retval;
}

/**
 * @brief  check class name folder
 * @param  class name folder path
 * @return Read success or not
 * @retval TRUE Read success
 * @retval FALSE Read failure
 */
BOOL ClassDependency::ClassFolderChecker(
	const string classFolderName
)
{
	HANDLE search;
	WIN32_FIND_DATA fd;
	string fileName;
	string::size_type first;
	string::size_type end;

	if(!SetCurrentDirectory(classFolderName.c_str()))
	{
		return TRUE;
	}

	/* get all file name */
	search = FindFirstFile("*.*", &fd );
	if( search == INVALID_HANDLE_VALUE )
	{
		return TRUE;
	}

	while(TRUE)
	{
		/* Read file name */
		fileName = fd.cFileName;

		/* find last [/] or [\] */
		first = fileName.find_last_of("/\\");
		if(first == string::npos)
		{
			first = 0;
		}

		/* find last [.] */
		end = fileName.find_last_of(".");
		if(end == string::npos)
		{
			end = fileName.size();
		}

		/* if file pass's class name is not exit yet */
		fileName = fileName.substr(first + 1, end - first - 1);

		if(m_useClassList.end() == find(m_useClassList.begin(), m_useClassList.end(), fileName))
		{
			m_useClassList.push_back(fileName);
			m_findClassList.push(fileName);
		}

		if( !FindNextFile( search, &fd ) )
		{
			break;
		}
	}

	FindClose( search );

	return TRUE;
}

/**
 * @brief  check .h is used files
 * @param  Header file name string
 * @return Read success or not
 * @retval TRUE Read success
 * @retval FALSE Read failure
 */
BOOL ClassDependency::HeaderFileChecker(
	const string headerFileName
)
{
	BOOL retval=FALSE;
	ifstream ifs;
	string textLine;
	string objectLine;
	string compareString;
	char *pToken;
	char pTextLine[DEPEND_FILE_MAX_LINE_STRING];

	// Check depend file exist
	ifs.open(m_filePath.c_str(), ios::in);

	if(ifs)
	{
		/* check depend file */
		while(!ifs.eof())
		{
			objectLine = "\\";
			textLine = "\\";

			/* Add line until end of line is '\' */
			while(textLine[textLine.size() - 1] == '\\')
			{
				if(getline(ifs, textLine))
				{
					objectLine = objectLine.substr(0, objectLine.size() - 1) + textLine;
				}
			}

			/* if find header name */
			if(objectLine.find(headerFileName) != string::npos)
			{
				retval = TRUE;

				/* Copy string to char buffer */
				strcpy(pTextLine, objectLine.c_str());

				/* Sepalate [:] */
				pToken = strtok(pTextLine, g_partition);
				compareString = pToken;

				/* delete space befor and after category */
				compareString = pafee_DeleteCharacter(compareString, g_delimiter);
				/* delete ".o" */
				compareString = compareString.substr(0, (compareString.size() - 2));

				if(m_useClassList.end() == find(m_useClassList.begin(), m_useClassList.end(), compareString))
				{
					m_useClassList.push_back(compareString);
					m_findClassList.push(compareString);
				}
			}
		}
	}
	else
	{
		cout << "Can not find dependency definition file\n";
	}

	if(retval == FALSE)
	{
		cout << "Can not find dependency. header: " << headerFileName << endl;
		retval = TRUE;
	}

	return retval;
}


#ifdef __cplusplus
}; // namespace Pafee
#endif
