//****************************************************
//
//      project     PAFEE
//!      @file:     StructureDefinition.cpp
//!      @author:   Technocrats
//!      @brief:    Structure Definition File Analizer class
//
//*****************************************************

#include "StdAfx.h"
#include <iostream>
#include <fstream>
#include "define.h"
#include "assert.h"
#include "StructureDefinition.h"

#ifdef __cplusplus
namespace Pafee
{
#endif

/**
 * @brief constractor of CommandOption Class
 */
StructureDefinition::StructureDefinition(
	string filePath
)
{
	m_FilePath = filePath;
}

/**
 * @brief destractor of CommandOption Class
 */
StructureDefinition::~StructureDefinition(
)
{
}

/**
 * @brief  Read Structure Definition File
 * @param  FilePath : Structure Definition File Path
 * @return Read success or not
 * @retval TRUE : Read success
 * @retval FALSE : Read failure
 */
BOOL StructureDefinition::ReadFile(
	const string filePath
)
{
	BOOL retval = FALSE;

	if(filePath.length() != 0)
	{
		m_FilePath = filePath;
		retval = TRUE;
	}

	return retval;
}

/**
 * @brief  Analyze Structure Definition File
 * @param  (none)
 * @return Analysis success or not
 * @retval TRUE : Analysis success
 * @retval FALSE : Analysis failure
 */
BOOL StructureDefinition::DefineStructure(
)
{
	BOOL retval = TRUE;

	ifstream fileStream;
	string lineString = "";
	string workString = "";
	int targetPosition;
	char count = 0;
	string categoryName = "";

	if(m_FilePath.empty())
	{
		m_FilePath = ".\\pafeeConfig.ini";
	}
	else
	{
		char last = m_FilePath.at(m_FilePath.length()-1);
		if((last == '/') || (last == '\\'))
		{
			m_FilePath += "pafeeConfig.ini";
		}
	}

	// open filestream
	fileStream.open(m_FilePath.c_str());
	if(!fileStream)
	{
		cout << "Can not find structure definition file\n";
		retval = FALSE;
	}
	else
	{
		while(!fileStream.eof())
		{
			// copy one line to buffer
			lineString = pafee_GetFileLine(fileStream);

			// delete strings after '#'
			targetPosition = lineString.find("#");
			if(targetPosition != lineString.npos)
			{
				lineString.erase(targetPosition, lineString.length()-targetPosition);
			}
			
#if 0
			// delete strings after '['
			targetPosition = lineString.find("[");
			if(targetPosition != lineString.npos)
			{
				lineString.erase(targetPosition, lineString.length()-targetPosition);
			}
#endif
			// convert tabs to spaces
			while((targetPosition = lineString.find("\t")) != lineString.npos)
			{
				lineString.replace(targetPosition, 1, " ");
			}
			
			lineString += " ";  // add space at the end of the line
			
			while(lineString.length() > 1)  // don't enter this roop when the string is NULL or 1 space only
			{
				// get string before space
				targetPosition = lineString.find(" ");
				if(targetPosition == 0)
				{
					lineString.erase(targetPosition, 1);  // erase one character(" ")
				}
				else
				{
					workString = lineString.substr(0, targetPosition);
					lineString.erase(0, targetPosition+1);
					
					if(workString.at(0) == '[')    // category name
					{
#if 1
						targetPosition = workString.find("]");
						if(targetPosition != workString.npos)
						{
							categoryName = workString.substr(1, targetPosition-1);
						}
#endif
					}
					else                           // class name or other
					{
#if 1
						if(categoryName != "")
						{
							workString = categoryName + "/" + workString;
							m_usingClassList.push_back(workString);
						}
						else
						{
							// do nothing
						}
#endif
					}
				}
			}
		}  // end of while(!fileStream.eof())
	}
	return retval;
}


/**
 * @brief  Return Class Information List
 * @param  pClassList : Pointer to Class Information List
 * @return Get success or not
 * @retval TRUE : Get success
 * @retval FALSE : Get failure
 */
BOOL StructureDefinition::GetClassList(
	list<string> *pClassList
)
{
	BOOL retval = FALSE;

	if(pClassList != NULL)
	{
		*pClassList = m_usingClassList;
		retval = TRUE;
	}

	return retval;
}

#ifdef __cplusplus
}; // namespace Pafee
#endif
