//********************************************************************
//
//! 	 @file: 	PafeeCoverageRecorder.h
//! 	 @author:	T.Miyawaki
//! 	 @brief:	Coverage recorder for testing
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _COVERAGE_RECORDER_H_INCLUDED_
#define _COVERAGE_RECORDER_H_INCLUDED_

namespace Pafee {

class CoverageRecorder
{

public:
	
	//! Constractor
	CoverageRecorder();
	
	//! Destoractor
	virtual ~CoverageRecorder();
	
	//! Record coverage
	void Record(int lineNumber);
	
	//! Show coverage
	void Show(void);
	
protected:
	enum { MAX_COVERAGE = 512};

	//! Sort data
	void SortData(void);

	//! Swap data
	void Swap(int& dataA, int& dataB);
	
	//! Coverage data
	struct
	{
		int lineNumber;		///< line number
		int count;	///< passed count
	} m_coverage[MAX_COVERAGE];

	int m_dataCount;		///< data count
	int m_overFlow;			///< over flow counter
};

};

#endif // _COVERAGE_RECORDER_H_INCLUDED_
