//****************************************************
//
//      project     PAFEE
//!      @file:     PafeeZip.h
//!      @author:   chu
//!      @brief:    Zip class header file
//
//*****************************************************

//
// Copyright (C) 1995-2003 Jean-loup Gailly.
// For conditions of distribution and use, see copyright notice in zlib.h
//

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _ZIP_H_INCLUDED_
#define _ZIP_H_INCLUDED_

#include <zip/zlib.h>
#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "pafee_string.h"

#ifdef PAFEE_UNITTEST
#include <assert.h>
#include <stdlib.h>
#endif //PAFEE_UNITTEST

namespace Pafee
{

//------------------------------------------------------------------------------
//!    - @brief    This class supports Compression and uncompression function 
//!    - @brief    according to GZip
//!    .
//------------------------------------------------------------------------------
class Zip : public Object
{
public:
	Zip(Allocator* pAllocator=NULL)
	{
		SetAllocator(pAllocator);
	}

	~Zip(){};
	bool Encode(void* pDestination, unsigned long* pDestinationLength, 
				const void* pSource, unsigned long sourceLength);
	bool Decode(void* pDestination, unsigned long* pDestinationLength, 
				const void* pSource, unsigned long sourceLength);

#ifdef PAFEE_UNITTEST
public: // unit test function
	static void UnitTest(void);
#endif // PAFEE_UNITTEST

};

};	// Pafee
#endif